@@ -1,6 +1,7 @@
 import json
 import logging
 import uuid
+import threading
 
 import math
 import sqlalchemy
@@ -84,6 +85,8 @@ class SqlAlchemyStore(AbstractStore):
 
     ARTIFACTS_FOLDER_NAME = "artifacts"
     DEFAULT_EXPERIMENT_ID = "0"
+    _db_uri_sql_alchemy_engine_map = {}
+    _db_uri_sql_alchemy_engine_map_lock = threading.Lock()
 
     def __init__(self, db_uri, default_artifact_root):
         """
@@ -101,7 +104,19 @@ class SqlAlchemyStore(AbstractStore):
         self.db_uri = db_uri
         self.db_type = extract_db_type_from_uri(db_uri)
         self.artifact_root_uri = default_artifact_root
-        self.engine = mlflow.store.db.utils.create_sqlalchemy_engine_with_retry(db_uri)
+        # Quick check to see if the respective SQLAlchemy database engine has already been created.
+        if db_uri not in SqlAlchemyStore._db_uri_sql_alchemy_engine_map:
+            with SqlAlchemyStore._db_uri_sql_alchemy_engine_map_lock:
+                # Repeat check to prevent race conditions where one thread checks for an existing
+                # engine while another is creating the respective one, resulting in multiple
+                # engines being created. It isn't combined with the above check to prevent
+                # inefficiency from multiple threads waiting for the lock to check for engine
+                # existence if it has already been created.
+                if db_uri not in SqlAlchemyStore._db_uri_sql_alchemy_engine_map:
+                    SqlAlchemyStore._db_uri_sql_alchemy_engine_map[
+                        db_uri
+                    ] = mlflow.store.db.utils.create_sqlalchemy_engine_with_retry(db_uri)
+        self.engine = SqlAlchemyStore._db_uri_sql_alchemy_engine_map[db_uri]
         # On a completely fresh MLflow installation against an empty database (verify database
         # emptiness by checking that 'experiments' etc aren't in the list of table names), run all
         # DB migrations
